/* Copyright 2022 Matthew J Wolf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * -----
 * Based on F62_-_HHKB_-_Split_Backspace.json from Joe of Model F Labs
 */

#include QMK_KEYBOARD_H

enum layer_names {
    _BASE,
    _CAP,
    _NUM,
    _FN1
};

enum custom_keycodes {
  TBMACRO = SAFE_RANGE,
  DGRMCRO,
  WRKMOD,
  NUMTEST,
  NUMTEST2,
  NUMTEST3
};

bool bnumlock = false;
bool bcapslock = false;
bool bscrollock = false;
bool numlock_changed = false;
bool workmode = true;
int ledcnt = 0;
short readvalue=0;
bool bledcnt = false;
bool bledcaps = false;
bool blednum = false;

void sendint(int sendint){
    bool bnum = host_keyboard_led_state().num_lock;
    bool bcap = host_keyboard_led_state().caps_lock;
    bool bscroll = host_keyboard_led_state().scroll_lock;
    if (bscroll){tap_code(KC_SCRL);}
    if (bcap){tap_code(KC_CAPS);}
    if (bnum){tap_code(KC_NUM);}
    int x = 0;
    tap_code(KC_SCRL);
    while(x<2){
        if(((sendint >> x) & 1) != 0){
            //tap_code(KC_1);
            tap_code(KC_CAPS);
            tap_code(KC_NUM);
            tap_code(KC_NUM);
            tap_code(KC_CAPS);
        } else {
            //tap_code(KC_0);
            tap_code(KC_CAPS);
            tap_code(KC_CAPS);
        }
        x++;
    }
    tap_code(KC_SCRL);
}

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
    [_BASE] = LAYOUT_all(
        KC_GRV,     KC_1,    KC_2,    KC_CAPS,       KC_4,   KC_5, LSFT(KC_8),
        KC_TAB,     KC_Q,    KC_W,    KC_E,       KC_R,   KC_T, RALT(KC_8),
        MO(_CAP),   KC_A,    KC_S,    KC_D,       KC_F,   KC_G,
        KC_LSFT,    KC_Z,    KC_X,    KC_C,       KC_V,   KC_B, RALT(KC_7),
        KC_LCTL, KC_LGUI, KC_LALT,  KC_NUBS, MO(_NUM),
                            KC_PGUP, KC_PGDN,
                    KC_SPC, KC_DEL, KC_HOME,
                    KC_SPC, KC_DEL, KC_END
    ),
    [_CAP] = LAYOUT_all(
        KC_GRV,     KC_1,    KC_2,    KC_3,     KC_4,   KC_5, _______,
        TBMACRO, KC_MPRV, KC_MPLY, KC_MNXT,  ALT_F4, KC_CAPS,   QK_BOOT,
        _______, _______, KC_VOLD, KC_VOLU, KC_MUTE, _______,
        KC_LSFT,    CTLZ,    CTLX,    CTLC,    DGRMCRO, _______,   _______,
        _______, _______, CTALDEL, _______, _______,
                            _______, _______,
                    _______, _______, _______,
                    _______, _______, QK_BOOT
    ),
    [_NUM] = LAYOUT_all(
        KC_TILD, KC_EXLM, KC_AT,   KC_HASH, KC_DLR,  KC_PERC, KC_CIRC,
        KC_TAB,    KC_F1,   KC_F2,   KC_F3,   KC_F4,  KC_ESC, QK_BOOT,
        _______, KC_F5,   KC_F6,   KC_F7,   KC_F8,  KC_INS,
        KC_LSFT,   KC_F9,  KC_F10,  KC_F11,  KC_F12,  KC_DEL,  KC_END,
        _______, _______, KC_LALT, _______, _______,
                            _______, _______,
                    _______, _______, _______,
                    _______, _______, QK_BOOT
    ),
    [_FN1] = LAYOUT_all(
        KC_GRV,  KC_1,    KC_2,    KC_3,    KC_4,    KC_5,    KC_6,
        KC_GRV,    KC_1,    KC_2,       KC_3,    KC_4,      KC_5,  KC_6,
        KC_EQL, S(KC_EQL),   RALT(KC_2), RALT(KC_3), RALT(KC_4), RALT(KC_E),
        KC_LSFT, KC_NUBS, RALT(KC_NUBS), S(KC_NUBS), DGRMCRO, _______, _______,
        _______, _______,       KC_RALT,    _______,_______,
                            _______, _______,
                    _______, _______, _______,
                    _______, _______, QK_BOOT
    )
};


bool process_record_user(uint16_t keycode, keyrecord_t *record) {
    if (record->event.pressed) {
        switch(keycode) {
            case TBMACRO:
                //SEND_STRING(SS_TAP(X_TAB) SS_DOWN(X_LSHIFT) SS_TAP(X_HOME) SS_UP(X_LSHIFT) SS_TAP(X_DELETE));
                tap_code(KC_TAB);
                register_code(KC_LSFT);
                tap_code(KC_HOME);
                unregister_code(KC_LSFT);
                tap_code(KC_DEL);
                return false;
            case DGRMCRO:
                if(!bnumlock) {
                    //register_code(KC_NLCK);
                    //unregister_code(KC_NLCK);
                    tap_code(KC_NUM);
                    bnumlock = true;
                }
                //SEND_STRING(SS_DOWN(X_LALT) SS_TAP(X_P0) SS_TAP(X_P1) SS_TAP(X_P7) SS_TAP(X_P6) SS_UP(X_LALT));
                //SEND_STRING(SS_DOWN(X_LALT) SS_TAP(X_KP_0) SS_TAP(X_KP_1) SS_TAP(X_KP_7) SS_TAP(X_KP_6) SS_UP(X_LALT) );
                register_code(KC_LALT);
                tap_code(KC_KP_0);
                tap_code(KC_KP_1);
                tap_code(KC_KP_7);
                tap_code(KC_KP_6);
                unregister_code(KC_LALT);
                return false; 
            case NUMTEST:
                sendint(4);
                return false;         
            case NUMTEST2:
                sendint(5);
                return false;      
            case NUMTEST3:
                sendint(7);
                return false;   
        }
        
    }
    return true;
}

layer_state_t layer_state_set_user(layer_state_t state) {
    switch (biton32(state)) {
        case _BASE:
            sendint(0);
            if(host_keyboard_led_state().caps_lock) {
                tap_code(KC_CAPS);
            };
            if(host_keyboard_led_state().num_lock) {
                tap_code(KC_NUM);
            };
            break;
        case _NUM:
            if(!host_keyboard_led_state().num_lock) {
                tap_code(KC_NUM);
            };
            break;
        case _CAP:
            if(!host_keyboard_led_state().caps_lock) {
                tap_code(KC_CAPS);
            };
            break;
        default: //  for any other layers, or the default layer
            if(host_keyboard_led_state().num_lock) {
                tap_code(KC_NUM);
            };
            if(host_keyboard_led_state().caps_lock) {
                tap_code(KC_CAPS);
            };     
            break;
    }
  return state;
}

void led_set_user(uint8_t usb_led){
    if (host_keyboard_led_state().scroll_lock){
        bscrollock = true;
        if(!bledcnt){
            ledcnt=0;
            readvalue = 0;
            bledcnt=true;
        }
        if (host_keyboard_led_state().caps_lock) {
            bledcaps = true; 
            if (host_keyboard_led_state().num_lock) {
                blednum = true;
            }
        } else if((!host_keyboard_led_state().caps_lock & (!host_keyboard_led_state().num_lock))) {
            if (bledcaps){
                if (blednum){
                    //tap_code(KC_1);
                    readvalue |= 1 << ledcnt;
                } else {
                    //tap_code(KC_0);           
                    readvalue |= 0 << ledcnt;
                }
                bledcaps=false;
                blednum=false;
                ledcnt++;
            }
        }
    } else if (bscrollock){
        bscrollock = false;
        bledcnt = false;
        bledcaps = false;
        blednum = false;
        if (ledcnt==2){
            if(readvalue==2){
                layer_on(3);
            }
            if(readvalue==0){
                layer_off(3);
            }
        }
        ledcnt=0;
        /*char strvalue[4];
        sprintf(strvalue, "%d", readvalue);
        send_string(strvalue);*/
    } else {
        if (host_keyboard_led_state().num_lock & !bnumlock) {
            bnumlock = true;

        } else if (bnumlock) {
            bnumlock = false;
        }
        if (host_keyboard_led_state().caps_lock & !bcapslock) {
            bcapslock = true;
        } else if (bcapslock) {
            bcapslock = false;
        }
        if (host_keyboard_led_state().scroll_lock & !bscrollock){
            bscrollock = true;
        } else if (bscrollock){
            bscrollock = false;
        }
    }
}